/*-*- mode:c;indent-tabs-mode:nil;c-basic-offset:2;tab-width:8;coding:utf-8 -*-│
│vi: set net ft=c ts=2 sts=2 sw=2 fenc=utf-8                                :vi│
╞══════════════════════════════════════════════════════════════════════════════╡
│ Copyright 2020 Justine Alexandra Roberts Tunney                              │
│                                                                              │
│ This program is free software; you can redistribute it and/or modify         │
│ it under the terms of the GNU General Public License as published by         │
│ the Free Software Foundation; version 2 of the License.                      │
│                                                                              │
│ This program is distributed in the hope that it will be useful, but          │
│ WITHOUT ANY WARRANTY; without even the implied warranty of                   │
│ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU             │
│ General Public License for more details.                                     │
│                                                                              │
│ You should have received a copy of the GNU General Public License            │
│ along with this program; if not, write to the Free Software                  │
│ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA                │
│ 02110-1301 USA                                                               │
╚─────────────────────────────────────────────────────────────────────────────*/
#include "libc/dce.h"
#include "libc/fmt/fmt.h"

struct StringScannerState {
  const unsigned char *s;
  size_t i;
};

static int vsscanfcb(void *arg) {
  int res;
  struct StringScannerState *state;
  state = arg;
  if ((res = state->s[state->i])) {
    state->i++;
  } else {
    res = -1;
  }
  return res;
}

/**
 * Decodes string.
 *
 * @see libc/fmt/vcscanf.h (for docs and implementation)
 * @note to offer the best performance, we assume small codebases
 *     needing vsscanf() won't need sscanf() too; and as such, there's
 *     a small code size penalty to using both
 */
int(vsscanf)(const char *str, const char *fmt, va_list va) {
  struct StringScannerState state = {(const unsigned char *)str, 0};
  return vcscanf(vsscanfcb, &state, fmt, va);
}
